/*
   ioat94k.h - internal register definitions for AT94K series FPSLIC(tm)

   Contributors:
     Created by Marek Michalkiewicz <marekm@linux.org.pl>

   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.  You may
   use, modify or distribute it freely.

   This code is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   Based on advance information Atmel datasheet Rev. 1138B-12/99.
 */

#ifndef __IOAT94K
#define __IOAT94K 1

#define AVR_MEGA 1
#define AVR_ENHANCED 1

/* I/O space addresses */

/* UART1 Baud Rate Register */
#define UBRR1	0x00

/* UART1 Control and Status Registers */
#define USCR1B	0x01
#define USCR1A	0x02

/* UART1 I/O Data Register */
#define UDR1	0x03

/* 0x04 reserved */

/* Input Pins, Port E */
#define PINE	0x05

/* Data Direction Register, Port E */
#define DDRE	0x06

/* Data Register, Port E */
#define PORTE	0x07

/* 0x08 reserved */

/* UART0 Baud Rate Register */
#define UBRR0	0x09

/* UART0 Control and Status Registers */
#define USCR0B	0x0A
#define USCR0A	0x0B

/* UART0 I/O Data Register */
#define UDR0	0x0C

/* 0x0D..0x0F reserved */

/* Input Pins, Port D */
#define PIND	0x10

/* Data Direction Register, Port D */
#define DDRD	0x11

/* Data Register, Port D */
#define PORTD	0x12

/* FPGA I/O Select Control Register */
#define FISCR	0x13

/* FPGA I/O Select Registers A, B, C, D */
#define FISUA	0x14
#define FISUB	0x15
#define FISUC	0x16
#define FISUD	0x17

/* FPGA Cache Logic(R) registers (top secret, under NDA) */
#define FPGAX	0x18
#define FPGAY	0x19
#define FPGAZ	0x1A
#define FPGAD	0x1B

/* TWI stands for "Two Wire Interface" or "TWI Was I2C(tm)" */

/* 2-wire Serial Bit Rate Register */
#define TWBR	0x1C

/* 2-wire Serial Status Register */
#define TWSR	0x1D

/* 2-wire Serial (Slave) Address Register */
#define TWAR	0x1E

/* 2-wire Serial Data Register */
#define TWDR	0x1F

/* UART Baud Register High */
#define UBRRH	0x20

/* Watchdog Timer Control Register */
#define WDTCR	0x21

/* Timer/Counter2 Output Compare Register */
#define OCR2	0x22

/* Timer/Counter2 (8-bit) */
#define TCNT2	0x23

/* Timer/Counter1 Input Capture Register Low Byte */
#define ICR1L	0x24

/* Timer/Counter1 Input Capture Register High Byte */
#define ICR1H	0x25

/* Asynchronous mode StatuS Register */
#define ASSR	0x26

/* Timer/Counter2 Control Register */
#define TCCR2	0x27

/* Timer/Counter1 Output Compare RegisterB Low Byte */
#define OCR1BL	0x28

/* Timer/Counter1 Output Compare RegisterB High Byte */
#define OCR1BH	0x29

/* Timer/Counter1 Output Compare RegisterA Low Byte */
#define OCR1AL	0x2A

/* Timer/Counter1 Output Compare RegisterA High Byte */
#define OCR1AH	0x2B

/* Timer/Counter1 Low Byte */
#define TCNT1L	0x2C

/* Timer/Counter1 High Byte */
#define TCNT1H	0x2D

/* Timer/Counter1 Control Register B */
#define TCCR1B	0x2E

/* Timer/Counter1 Control Register A */
#define TCCR1A	0x2F

/* Special Function IO Register */
#define SFIOR	0x30

/* Timer/Counter0 Output Compare Register */
#define OCR0	0x31

/* Timer/Counter0 (8-bit) */
#define TCNT0	0x32

/* Timer/Counter0 Control Register */
#define TCCR0	0x33

/* 0x34 reserved */

/* MCU Control/Status Register */
#define MCUR	0x35

/* 2-wire Serial Control Register */
#define TWCR	0x36

/* 0x37 reserved */

/* Timer/Counter Interrupt Flag Register */
#define TIFR	0x38

/* Timer/Counter Interrupt MaSK Register */
#define TIMSK	0x39

/* Software Control Register */
#define SFTCR	0x3A

/* External Interrupt Mask/Flag Register */
#define EIMF	0x3B

/* 0x3C reserved */

/* Stack Pointer Low */
#define SPL	0x3D

/* Stack Pointer High */
#define SPH	0x3E

/* Status REGister */
#define SREG	0x3F

/* Interrupt vectors (byte addresses) */

#define RESET_vect		(0x00)
#define FPGA_INT0_vect		(0x04)
#define INT0_vect		(0x08)
#define FPGA_INT1_vect		(0x0C)
#define INT1_vect		(0x10)
#define FPGA_INT2_vect		(0x14)
#define INT2_vect		(0x18)
#define FPGA_INT3_vect		(0x1C)
#define INT3_vect		(0x20)
#define TIMER2_COMP_vect	(0x24)
#define TIMER2_OVF_vect		(0x28)
#define TIMER1_CAPT_vect	(0x2C)
#define TIMER1_COMPA_vect	(0x30)
#define TIMER1_COMPB_vect	(0x34)
#define TIMER1_OVF_vect		(0x38)
#define TIMER0_COMP_vect	(0x3C)
#define TIMER0_OVF_vect		(0x40)
#define FPGA_INT4_vect		(0x44)
#define FPGA_INT5_vect		(0x48)
#define FPGA_INT6_vect		(0x4C)
#define FPGA_INT7_vect		(0x50)
#define UART_RX_vect		(0x54)
#define UART_UDRE_vect		(0x58)
#define UART_TX_vect		(0x5C)
#define FPGA_INT8_vect		(0x60)
#define FPGA_INT9_vect		(0x64)
#define FPGA_INT10_vect		(0x68)
#define FPGA_INT11_vect		(0x6C)
#define UART1_RX_vect		(0x70)
#define UART1_UDRE_vect		(0x74)
#define UART1_TX_vect		(0x78)
#define FPGA_INT12_vect		(0x7C)
#define FPGA_INT13_vect		(0x80)
#define FPGA_INT14_vect		(0x84)
#define FPGA_INT15_vect		(0x88)
#define TWI_INT_vect		(0x8C)

#define INT_VECT_SIZE (0x90)

/* Bit numbers - XXX not here yet */

/* Pointer registers (same for all AVR devices so far) */
#define XL r26
#define XH r27
#define YL r28
#define YH r29
#define ZL r30
#define ZH r31

/*
   Last memory addresses - depending on configuration, it is possible
   to have 20K-32K of program memory and 4K-16K of data memory
   (all in the same 36K total of SRAM, loaded from external EEPROM).
 */

#ifndef RAMEND
#define RAMEND 0x0FFF
#endif

#define E2END 0

#ifndef FLASHEND
#define FLASHEND 0x7FFF
#endif

#endif  /* __IOAT94K */
